#!/usr/bin/env pybricks-micropython
from pybricks.hubs import EV3Brick
from pybricks.ev3devices import (Motor, TouchSensor, ColorSensor,
                                 InfraredSensor, UltrasonicSensor, GyroSensor)
from pybricks.parameters import Port, Stop, Direction, Button, Color
from pybricks.tools import wait, StopWatch, DataLog
from pybricks.robotics import DriveBase
from pybricks.media.ev3dev import SoundFile, ImageFile
import math

# Create your objects here.
ev3 = EV3Brick()
#ev3.speaker.beep()

def drive(dir, speed, LM, RM):
    if dir > 100: dir = 100
    if dir < -100: dir = -100

    L = speed
    R = speed

    if dir > 0: R = speed*(1 - dir/50)
    else: L = speed*(1 + dir/50)

    LM.run(L)
    RM.run(R)

def driveMaze(dir, speed, LM, RM):
    if dir > 30: dir = 30
    if dir < -30: dir = -30

    L = speed
    R = speed

    if dir > 0: R = speed*(1 - dir/50)
    else: L = speed*(1 + dir/50)

    LM.run(L)
    RM.run(R)

def priamo(speed, distance, stop = True):
    distance = distance / (3.14 * 6.7) * 360
    k = -1
    o = R.angle()
    p = L.angle()
    if speed > 0:
        while ((R.angle() - o) + (L.angle() - p)) < distance*2:
            #print((R.angle() - o) + (L.angle() - p))
            error = (o - R.angle()) - (p - L.angle())
            dir = error*k
            drive(dir, speed, L, R)
    else:
        k = 1
        while ((R.angle() - o) + (L.angle() - p)) > distance*-2:
            #print((R.angle() - o) + (L.angle() - p))
            error = (o - R.angle()) - (p - L.angle())
            dir = error*k
            drive(dir, speed, L, R)
    if stop:
        L.stop()
        R.stop()

def povorot(speed, angle, stop = True):
    Dr = 12.5
    Dk = 6.7
    o = R.angle()
    p = L.angle()
    if angle > 0:
        while ((o - R.angle()) + (L.angle() - p)) < 2*angle*Dr/Dk:
            drive(100, speed, L, R)
    else:
        while ((o - R.angle()) + (L.angle() - p)) > 2*angle*Dr/Dk:
            drive(-100, speed, L, R)

    if stop:
        L.stop()
        R.stop()

R = Motor(Port.C)
L = Motor(Port.B)
Z = Motor(Port.A)

UsFront = UltrasonicSensor(Port.S1)
UsLeft = UltrasonicSensor(Port.S4)
                                                                                                                                                 
CsRight = ColorSensor(Port.S2)
CsLeft = ColorSensor(Port.S3)

wait(5000)

o = R.angle()
p = L.angle()

distance = 2900
while ((R.angle() - o) + (L.angle() - p)) < distance*2:
    u = CsRight.reflection()
    y = CsLeft.reflection()

    k = 1.1
    i = (u-y)*k
    drive(i, 250, R, L)

R.stop()
L.stop()
wait(500)

povorot(300, 88)
priamo(300, 23)
povorot(300, -90)
priamo(300, 71)
povorot(300, -90)

priamo(300, 8)
while True:
    drive(0, 300, L, R)
    f = CsRight.reflection()
    g = CsLeft.reflection()
    if f < 10 or g < 10:
        R.stop() 
        L.stop()
        break
priamo(300, 3)
povorot(300, 75)

o = R.angle()
p = L.angle()


while True:
    u = CsRight.reflection()
    y = CsLeft.reflection()

    k = 1.1
    i = (u-y)*k
    drive(i, 250, R, L)
    if u < 10 and y < 10:
        R.stop() 
        L.stop()
        break

wait(5000)



while True:
    UsLeftWalue = UsLeft.distance()
    UsFrontWalue = UsFront.distance()

    negr3 = CsRight.reflection()
    negr4 = CsLeft.reflection()

    if negr3 < 10 and negr4 < 10:
        break

    if UsFrontWalue < 100:
        R.run_angle(300, 200, wait=False)
        L.run_angle(-300, 180)
    else:
        error = 80 - UsLeftWalue
        dir = error * -1.5

        driveMaze(dir, 300, L, R)

R.stop() 
L.stop()

wait(5000)


def lineFollow(speed, distance, type):
    distance = distance / (3.14 * 6.7) * 360
    o = R.angle()
    p = L.angle()

    while ((R.angle() - o) + (L.angle() - p)) < distance*2:
        u = CsRight.reflection()
        y = CsLeft.reflection()

        k = 1.1 * type
        i = (u-y)*k
        drive(i, speed, R, L)


lineFollow(250, 8, 1)
priamo(250, 5, stop = False)
lineFollow(250, 22, -1)
priamo(250, 5, stop = False)
lineFollow(250, 22, 1)
priamo(250, 5, stop = False)
lineFollow(250, 34, -1)
R.run_angle(250, 80, wait=False)
L.run_angle(250, 150)

lineFollow(250, 26, 1)
R.run_angle(250, 120, wait=False)
L.run_angle(250, 220)

lineFollow(250, 22, -1)
priamo(250, 3, stop = False)
lineFollow(250, 21, 1)
L.run_angle(250, 120, wait=False)
R.run_angle(250, 220)
lineFollow(250, 23, -1)
L.run_angle(250, 120, wait=False)
R.run_angle(250, 220)
while True:
    u = CsRight.reflection()
    y = CsLeft.reflection()

    k = 1.1
    i = (u-y)*k
    drive(i, 250, R, L)
    if u < 10 and y < 10:
        R.stop() 
        L.stop()
        break



Z.run_angle(300, 250)
priamo(250, 8, False)
L.run(0)
R.run_angle(235, 120)
priamo(250, 100, False)
L.run(0)
R.run(0)
Z.run_angle(-300, 200)
Z.run_angle(300, 250)
povorot(250, 90, False)
priamo(250, 60, False)
L.run(0)
R.run(0)
Z.run_angle(-300, 200)
Z.run_angle(300, 250)
povorot(250, 90, False)
priamo(250, 68, False)
L.run(0)
R.run(0)
Z.run_angle(-300, 200)
Z.run_angle(300, 250)
povorot(250, 100, False)
priamo(250, 60, False)
L.run(0)
R.run(0)
Z.run_angle(-300, 200)
Z.run_angle(300, 200)
priamo(-300, 5)
povorot(300, 125)
priamo(250, 25)
Z.run_angle(-300, 400)



povorot(300, 45)

nigga = CsRight.reflection()
nigga1 = CsLeft.reflection()             

if nigga < 10 and nigga1 < 10:
    L.run_stop()
    R.run_angle(300, 160)

o = R.angle()
p = L.angle()

distance = 100
while ((R.angle() - o) + (L.angle() - p)) < distance*2:
    u = CsRight.reflection()
    y = CsLeft.reflection()

    k = 1
    i = (u-y)*k
    drive(i, 300, R, L)

povorot(300, 90)

priamo(300, 50)

L.stop()
R.stop()












'''

































while True:
    UsLeftWalue = UsLeft.distance()
    UsFrontWalue = UsFront.distance()

    negr3 = CsRight.reflection()
    negr4 = CsLeft.reflection()

    if negr3 < 10 and negr4 < 10:
        break

    if UsFrontWalue < 120:
        R.run_angle(-300, 200, wait=False)
        L.run_angle(300, 200)
    else:
        error = 80 - UsLeftWalue
        dir = error * 1.5

        driveMaze(dir, 300, L, R)





R.run_angle(300, 550)
L.run_angle(300, 550)

R.run_angle(300, -350)
L.run_angle(300, 350)





k = 1,5
o = R.angle()
p = L.angle()
while ((o - R.angle()) + (p - L.angle())) < 1000:
    u = CsRight.reflection()
    y = CsLeft.reflection()

    i = (u-y)*k
    drive(i, 300, R, L)


















    


































LM = Motor(Port.B, Direction.COUNTERCLOCKWISE)
RM = Motor(Port.C, Direction.COUNTERCLOCKWISE)
CrM = Motor(Port.A)

def drive(dir, speed, LM, RM):
    if dir > 50: dir = 50
    if dir < -50: dir = -50

    L = speed
    R = speed

    if dir > 0: R = speed*(1 - dir/50)
    else: L = speed*(1 + dir/50)

    LM.run(L)
    RM.run(R)

#CrM.run_angle(100, 500)

# датчик 1 - левый датчик, датчик 2 - центральный, датчик 3 - правый
S1 = ColorSensor(Port.S1)
S2 = ColorSensor(Port.S2)
S3 = ColorSensor(Port.S3)

kp = 2
ki = 0#.05
sum = 0
lastLine = 1


while True:
    V1 = S1.reflection()
    V2 = S2.reflection()
    V3 = S3.reflection()

    error = V1 - V3*1.2

    sum = sum * 0.8 + error

    P = error * kp
    I = sum * ki

    dir = P + I

    if V1 < 10:
        lastLine = -1
    elif V3 < 10:
        lastLine = 1

    if (V1 < 10) or (V2 < 10) or (V2 < 10):
        drive(dir, 600, LM, RM)
    else:
        drive(35 * lastLine, 600, LM, RM)





























LM = Motor(Port.B)
RM = Motor(Port.C)
U = GyroSensor(Port.S1)

i = 0
g = 0

x = 0
y = 0

while True:
    r = LM.angle() - i
    f = RM.angle() - g

    i = LM.angle()б
    g = RM.angle()
    h = U.angle()
    q = (r + f)/2

    x = q*math.sin(math.radians(h)) + x
    y = q*math.cos(math.radians(h)) + y

    print(x,y)
    

































LM = Motor(Port.B)
RM = Motor(Port.C)

l = 3000
startSpeed   = 50
maxSpeed     = 800
minSpeed     = 0.1
acceleration = 1

l1 = (maxSpeed - startSpeed)/acceleration
l2 = l - (maxSpeed - minSpeed)/acceleration
print(l1, l2)
while LM.angle() < l:
    speed = 0
    if LM.angle() < l1:
        speed = acceleration * LM.angle() + startSpeed
    elif LM.angle() > l2:
        speed = maxSpeed - acceleration * (LM.angle() - l2)
    else:
        speed = maxSpeed

    print(speed)
    
    LM.run(speed)
    RM.run(speed)























CS = ColorSensor(Port.S1)
YS1 =  UltrasonicSensor(Port.S2)
YS2 =  UltrasonicSensor(Port.S3)
GS = GyroSensor(Port.S4)

g = 0

# Функция рулевого управления
def drive(dir, speed, LM, RM):
    if dir > 100: dir = 100
    if dir < -100: dir = -100

    L = speed
    R = speed

    if dir > 0: R = speed*(1 - dir/50)
    else: L = speed*(1 + dir/50)

    LM.run(L)
    RM.run(R)


# функция выгрузки шариков
def kyzov():
    SM.run_angle(-800, 7*360, wait=True)
    wait(1000)
    SM.run_angle(800, 7*360, wait=False)


# функция загрузки шариков
def dolbit():
    for i in range(4):
        LM.run_time(800, 1200, wait=False)
        RM.run_time(800, 1200, wait=True)
        LM.run_angle (-600, 180, wait=False)
        RM.run_angle (-600, 180, wait=True)


# функция что бы проехать 1 ячейку прямо
# параметр viravnivanie отвечает за выравнивание о переднюю стену
def runForward(viravnivanie = False, magik = False, dist = 45):
    global g
    LM.reset_angle(0)
    RM.reset_angle(0)
    wait(50)

    x       = 1100
    x_start = 200 
    x_stop  = 200

    start_speed = 800
    stop_speed  = 450
    max_speed   = 800

    L = 0
    d1 = YS1.distance()
    d2 = YS2.distance()
    while L < 1100 and d1 > dist:
        d1 = YS1.distance()
        d2 = YS2.distance()
        L = abs((int(RM.angle()) + int(LM.angle()))/2)

        speed = 0
        if L <= x_start:
            speed = start_speed + (max_speed - start_speed)*(L/x_start)
        elif L <= x - x_stop:
            speed = max_speed
        else:
            if magik and d1 > 300 and d2 < 200:
                speed = max_speed
            else:
                speed = max_speed - (max_speed - stop_speed)*((L - (x - x_stop))/x_stop)

        d = (g - GS.angle())*5
        if d2 < 150:
            d = (104 - d2)*0.3
        
        drive(d, speed*-1, RM, LM) 

    LM.stop(Stop.BRAKE)
    RM.stop(Stop.BRAKE)
    wait(100)

    if viravnivanie and YS1.distance() < 200:
        X = (YS1.distance() - dist)*1146/300
        LM.run_angle (-300, X, wait=False)
        RM.run_angle (-300, X, wait=True)


# функция для поворота налево
# 
def turnLeft(): 
    global g
    ang       = 90
    ang_start = 15 
    ang_stop  = 30

    start_speed = 300
    stop_speed  = 150
    max_speed   = 500

    L = 0
    while L < 90:
        L = g - GS.angle()
        speed = 0
        if L <= ang_start:
            speed = start_speed + (max_speed - start_speed)*(L/ang_start)
        elif L <= ang - ang_stop:
            speed = max_speed
        else:
            speed = max_speed - (max_speed - stop_speed)*((L - (ang - ang_stop))/ang_stop)
        
        drive(-100, speed, LM, RM)

    g = g - 90

    LM.stop(Stop.BRAKE)
    RM.stop(Stop.BRAKE)
    wait(100)


# функция для поворота направо
# параметр viravnivanie отвечает за выравнивание о заднюю стену после поворота
def turnRight(viravnivanie = False):
    global g
    ang       = 90
    ang_start = 15
    ang_stop  = 30

    start_speed = 300
    stop_speed = 150
    max_speed = 800

    L = 0
    while L < 90:
        L = GS.angle() - g
        speed = 0
        if L <= ang_start:
            speed = start_speed + (max_speed - start_speed)*(L/ang_start)
        elif L <= ang - ang_stop:
            speed = max_speed
        else:
            speed = max_speed - (max_speed - stop_speed)*((L - (ang - ang_stop))/ang_stop)
        
        drive(100, speed, LM, RM)

    g = g + 90

    LM.stop(Stop.BRAKE)
    RM.stop(Stop.BRAKE)
    wait(100)

    if viravnivanie:
        LM.run_time(800, 1200, wait=False)
        RM.run_time(800, 1200, wait=True)
        GS.reset_angle(0)
        g = 0
        LM.run_angle (-400, 208, wait=False)
        RM.run_angle (-400, 208, wait=True)


#ev3.speaker.beep() # - бип 

# загрузка шариков
dolbit()

dir = 8  # Стартовое направление робота
way = [] # переменна для пути робота до финиша (в начале он пуст)

# Цикл в процессе которого робот двигается по правилу левой руки к финишу
# В процессе поиска финиша робот записывает путь
while CS.color() != Color.RED:
    if YS2.distance() > 150:
        turnLeft()
        dir = dir*2%10 # поворот направления налево из 8 делает 4 и т.д.

        runForward(True, True)
        way.append(dir) # добавляем движение робота в путь
    else:
        if YS1.distance() < 200:
            turnRight(True)
            dir = dir*3%10 # поворот направления направо из 8 делает 6 и т.д.
        else:
            runForward(True, True)
            way.append(dir)    


# расчёт кратчайшего пути
flag = True
while flag:
    flag = False
    x = len(way)-1
    for i in range(x):
        if way[i] + way[i+1] == 10:
            way.pop(i)
            way.pop(i)
            flag = True
            break


# создаём обратный путь
way2 = []

# переписывем путь читая его с конца списка
for i in range(len(way)):
    way2.append(way[(i+1)*-1])

# меняем направления движения в пути для движения от финиша к старту
for i in range(len(way2)):
    if way2[i] == 8:
        way2[i] = 2
    elif way2[i] == 2:
        way2[i] = 8
    elif way2[i] == 6:
        way2[i] = 4
    elif way2[i] == 4:
        way2[i] = 6


# тут робот доехал до финиша и расчитал пути
# все его дальнейшие действия будут повторятся сколько угодно раз для доставки шариков
while True:
    # разгрузка
    turnLeft()
    LM.run_angle (300, 200, wait=False)
    RM.run_angle (300, 200, wait=True)
    kyzov()
    LM.run_angle (-300, 200, wait=False)
    RM.run_angle (-300, 200, wait=True)
    
    turnLeft()
    LM.run_time(600, 1200, wait=False)
    RM.run_time(600, 1200, wait=True)
    GS.reset_angle(0)
    g = 0
    LM.run_angle (-300, 208, wait=False)
    RM.run_angle (-300, 208, wait=True)
    dir =  dir*4%10

    # едем от финиша к старту
    for i in range(len(way2)):
        dist = 45
        if i == len(way2) - 1:
            dist = 60
        
        if way2[i] == dir:
            runForward(True, dist = 45)
        elif way2[i] == dir*3%10:
            turnRight()
            runForward(True, dist = 45)
            dir = way2[i]
        elif way2[i] == dir*2%10:
            turnLeft()
            runForward(True, dist = 45)
            dir = way2[i]

    # загрузка
    turnRight()
    LM.run_time(600, 1200, wait=False)
    RM.run_time(600, 1200, wait=True)
    GS.reset_angle(0)
    g = 0
    LM.run_angle (-300, 208, wait=False)
    RM.run_angle (-300, 208, wait=True)
    dir =  dir*4%10
    turnRight()
    dolbit()

    # едем от старта к финишу
    for i in range(len(way)):
        if way[i] == dir:
            runForward(True)
        elif way[i] == dir*3%10:
            turnRight()
            runForward(True)
            dir = way[i]
        elif way[i] == dir*2%10:
            turnLeft()
            runForward(True)
            dir = way[i]
            
'''